#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import functools
import os

import requests
from gql import Client, gql, transport
from gql.dsl import DSLMutation, DSLSchema, dsl_gql
from gql.transport.exceptions import TransportQueryError
from gql.transport.requests import RequestsHTTPTransport
from gql.transport.requests import log as requests_logger
from gql.transport.websockets import WebsocketsTransport
from gql.transport.websockets import log as websockets_logger
from graphql import ExecutionResult
from urllib3.exceptions import MaxRetryError

from threat_db.logger import LOG, WARNING
from threat_db.queries import (
    auth_user_query,
    drop_all_query,
    health_query,
    introspect_query,
)

# GraphQL schema
graphql_schema = open(
    os.path.join(os.path.dirname(__file__), "base-schema.graphql")
).read()

requests_logger.setLevel(WARNING)
websockets_logger.setLevel(WARNING)

headers = {"Content-Type": "application/json", "Accept-Encoding": "gzip"}

# Add access tokens to the requests
# The kind people at dgraph use several different headers for tokens!
if os.getenv("DGRAPH_API_KEY"):
    headers["X-Dgraph-AuthToken"] = os.getenv("DGRAPH_API_KEY")
    headers["X-Auth-Token"] = os.getenv("DGRAPH_API_KEY")
if os.getenv("DGRAPH_CLOUD_API_KEY"):
    headers["Authorization"] = f"""Bearer {os.getenv("DGRAPH_CLOUD_API_KEY")}"""
if os.getenv("DGRAPH_ACL_KEY"):
    headers["X-Dgraph-AccessToken"] = os.getenv("DGRAPH_ACL_KEY")


def catch_db_errors(fn):
    @functools.wraps(fn)
    def caller(*args, **kwargs):
        try:
            return fn(*args, **kwargs)
        except requests.exceptions.ConnectionError:
            LOG.warn(
                "GraphQL API is unavailable. Please contact the API provider for support."
            )
        except MaxRetryError:
            LOG.warn("GraphQL and admin API are currently unavailable.")
        except TransportQueryError as tqe:
            if tqe.errors:
                first_error = tqe.errors[0].get("message", "")
                # dgraph doesn't support multiple concurrent mutations. So we workaround by retrying
                # See https://discuss.dgraph.io/t/transactions-in-graphql/6861
                if (
                    "couldn't commit transaction" in first_error
                    and "Please retry" in first_error
                ):
                    LOG.info("Retrying failed mutation")
                    return fn(*args, **kwargs)
            raise tqe
        except Exception as ex:
            LOG.exception(ex)

    return caller


def process_query_response(json_resp):
    if isinstance(json_resp, ExecutionResult):
        return {"data": json_resp.data, "errors": json_resp.errors}
    if isinstance(json_resp, dict) and json_resp.get("errors"):
        errors = json_resp.get("errors")
        if len(errors):
            first_error = errors[0].get("message")
            if "No Auth Token" in first_error:
                LOG.warn(
                    f"Please set the database authentication token via the environment variable DGRAPH_API_KEY"
                )
        LOG.error(json_resp.get("errors"))
        return json_resp.get("errors")
    elif isinstance(json_resp, list):
        return json_resp
    return json_resp


# Drop All - discard all data and start from a clean slate.
def drop_all(client, host):
    if host.endswith("graphql"):
        host = host.replace("/graphql", "") + "/alter"
    res = requests.post(host, data=drop_all_query, headers=headers)
    if res.ok:
        return process_query_response(res.json())
    LOG.warn(f"Drop all data failed due to {res.status_code} {host}")
    return None


def healthcheck(client, host):
    if host.endswith("graphql"):
        host = host.replace("/graphql", "") + "/health"
    res = requests.post(host, data=health_query, headers=headers)
    if res.ok:
        return process_query_response(res.json())
    LOG.warn(f"Unable to perform healthcheck due to {res.status_code} {host}")
    return None


def is_alive(client, host):
    health_res = healthcheck(client, host)
    for node in health_res:
        if node.get("instance") == "alpha" and node.get("status") == "healthy":
            return True
    return False


def create_schemas(client, host):
    needs_creation = True
    # Do not recreate schema unnecessarily
    try:
        with client as session:
            query = gql(introspect_query)
            result = session.execute(query)
            if result and result.get("__schema", {}).get("types"):
                needs_creation = False
                return result
    except requests.exceptions.ConnectionError:
        needs_creation = False
    except MaxRetryError:
        needs_creation = False
    except TransportQueryError as e:
        if e.errors:
            first_error = e.errors[0].get("message", "")
            needs_creation = "Not resolving __schema" in first_error
    except Exception as e:
        needs_creation = True
    if needs_creation:
        if host.endswith("graphql"):
            host = host.replace("/graphql", "") + "/admin/schema"
        res = requests.post(host, data=graphql_schema, headers=headers)
        if res.ok:
            return process_query_response(res.json())
        LOG.warn(f"Create schema failed due to {res.status_code} {host}")
        return None
    else:
        LOG.debug("Database schema already exists")
        return ""


@catch_db_errors
def create_bom(client, bom):
    with client as session:
        ds = DSLSchema(client.schema)
        query = dsl_gql(
            DSLMutation(
                ds.Mutation.addBom(input=bom, upsert=False).select(
                    ds.AddBomPayload.bom.select(ds.Bom.serialNumber)
                )
            )
        )
        try:
            result = session.execute(query)
        except TransportQueryError as tqe:
            LOG.exception(tqe)
            if tqe.errors:
                first_error = tqe.errors[0].get("message", "")
                if "duplicate XID found" in first_error:
                    LOG.info(
                        "Some components in this BoM were also found in another BoM."
                    )
                    return None
                elif (
                    "already exists for field serialNumber inside type Bom"
                    in first_error
                ):
                    LOG.info(
                        "BoM is immutable therefore duplicate submissions are not allowed."
                    )
                    return None
                elif "Non-nullable field 'serialNumber'" in first_error:
                    LOG.info("Issues found with nested serialNumber fields.")
                    return None
                else:
                    LOG.exception(tqe)
                    return None
        return process_query_response(result)


@catch_db_errors
def create_components(client, components):
    with client as session:
        ds = DSLSchema(client.schema)
        query = dsl_gql(
            DSLMutation(
                ds.Mutation.addComponent(input=components, upsert=True).select(
                    ds.AddComponentPayload.component.select(
                        ds.Component.purl, ds.Component.bomRef
                    )
                )
            )
        )
        result = session.execute(query)
        return process_query_response(result)


@catch_db_errors
def create_vulns(client, vulns):
    with client as session:
        ds = DSLSchema(client.schema)
        query = dsl_gql(
            DSLMutation(
                ds.Mutation.addVulnerability(input=vulns, upsert=True).select(
                    ds.AddVulnerabilityPayload.vulnerability.select(
                        ds.Vulnerability.id,
                        ds.Vulnerability.bomRef,
                        ds.Vulnerability.version,
                    )
                )
            )
        )
        result = session.execute(query)
        return process_query_response(result)


@catch_db_errors
def create_user(client, users):
    with client as session:
        ds = DSLSchema(client.schema)
        query = dsl_gql(
            DSLMutation(
                ds.Mutation.addUser(input=users, upsert=False).select(
                    ds.AddUserPayload.user.select(
                        ds.User.id, ds.User.created, ds.User.disabled
                    )
                )
            )
        )
        result = session.execute(query)
        return process_query_response(result)


@catch_db_errors
def auth_user(client, user_id, password):
    with client as session:
        query = gql(auth_user_query)
        result = session.execute(
            query, variable_values={"user_id": user_id, "password": password}
        )
        if result.get("checkUserPassword"):
            return True
    return False


def raw_execute(client, user_query):
    with client as session:
        q = user_query.get("query")
        query = gql(q)
        variables = user_query.get("variables", {})
        result = session.execute(query, variable_values=variables)
        return process_query_response(result)


def get(host, api_key=None):
    if not host.endswith("graphql"):
        host = f"{host}/graphql"
    if api_key:
        headers["Authorization"] = api_key
    client = None
    # if host.startswith("https"):
    #     wss_host = host.replace("https://", "wss://")
    #     try:
    #         transport = WebsocketsTransport(url=wss_host)
    #         client = Client(transport=transport, fetch_schema_from_transport=True)
    #     except Exception as ex:
    #         print(ex)
    if not client:
        transport = RequestsHTTPTransport(
            url=host, verify=True, headers=headers, retries=1
        )
        client = Client(transport=transport, fetch_schema_from_transport=True)
    return transport, client



import os
from datetime import timedelta
from tempfile import TemporaryDirectory

from flask import Flask, jsonify, request
from flask_jwt_extended import (
    JWTManager,
    create_access_token,
    get_jwt_identity,
    jwt_required,
)
from uwsgidecorators import filemon

import threat_db.graphclient as graph_client
import threat_db.loader as data_loader
from threat_db.config import JWT_ACCESS_TOKEN_EXPIRES_HOURS
from threat_db.logger import LOG

MAX_CONTENT_LENGTH = 10 * 1000 * 1000  # 10 Mb
ALLOWED_EXTENSIONS = ["json", "jsonl"]

DGRAPH_GRAPHQL_HOST = os.getenv("DGRAPH_GRAPHQL_HOST", "http://alpha:8080/graphql")
DGRAPH_RPC_HOST = os.getenv("DGRAPH_RPC_HOST", "alpha:9080")
THREATDB_DATA_DIR = os.getenv("THREATDB_DATA_DIR")

headers = {"Content-Type": "application/json", "Accept-Encoding": "gzip"}

app = Flask(__name__)
app.config["MAX_CONTENT_LENGTH"] = MAX_CONTENT_LENGTH
if os.getenv("JWT_SECRET_KEY"):
    app.config["JWT_SECRET_KEY"] = os.getenv("JWT_SECRET_KEY")
else:
    app.config["JWT_SECRET_KEY"] = os.urandom(64).hex()
app.config["JWT_TOKEN_LOCATION"] = ["headers", "json"]
app.config["JWT_ACCESS_TOKEN_EXPIRES"] = timedelta(hours=JWT_ACCESS_TOKEN_EXPIRES_HOURS)
if os.getenv("THREATDB_TMP_DIR"):
    app.config["UPLOAD_FOLDER"] = os.getenv("THREATDB_TMP_DIR")
else:
    app.config["UPLOAD_FOLDER"] = TemporaryDirectory(prefix="threatdb")

jwt = JWTManager(app)

transport, client = graph_client.get(DGRAPH_GRAPHQL_HOST, os.getenv("DGRAPH_API_KEY"))


def allowed_file(filename):
    return "." in filename and filename.rsplit(".", 1)[1].lower() in ALLOWED_EXTENSIONS


@app.route("/login", methods=["POST"])
def login():
    user_id = request.json.get("username", None)
    password = request.json.get("password", None)
    try:
        auth_res = graph_client.auth_user(client, user_id, password)
        if auth_res:
            access_token = create_access_token(identity=user_id)
            return jsonify(access_token=access_token)
        else:
            return jsonify({"msg": "Invalid user id or password"}), 401
    except Exception:
        return jsonify({"msg": "Invalid user id or password"}), 401


def identity(payload):
    user_id = payload["identity"]
    return user_id


@app.route("/healthcheck")
def healthcheck():
    if graph_client.is_alive(client, DGRAPH_GRAPHQL_HOST):
        return "ok"
    return ""


def process_file(file):
    if file.filename == "":
        return {"success": "false", "message": "Empty file uploaded"}, 500
    if file and not allowed_file(file.filename):
        return {
            "success": "false",
            "message": f"File is not a supported type. Supported types are {', '.join(ALLOWED_EXTENSIONS)}",
        }, 500
    result = data_loader.process_vex_file(client, file.stream)
    return result


if DGRAPH_GRAPHQL_HOST in ("http://alpha:8080/graphql"):

    @app.route("/whoami", methods=["GET"])
    @jwt_required()
    def whoami():
        current_user = get_jwt_identity()
        return jsonify(user=current_user)


@app.route("/import", methods=["POST"])
@jwt_required()
def import_data():
    current_user = get_jwt_identity()
    files = request.files.getlist("file")
    if not files:
        return {
            "success": "false",
            "message": "Upload files to import using the file attribute",
        }, 500
    # check if the post request has the file part
    for uf in files:
        result = process_file(uf)
        if not result:
            return {
                "success": "false",
                "message": "File was not processed successfully",
            }, 500
    return jsonify(success=True)


@app.route("/graphql", methods=["POST"])
@jwt_required()
def proxy_graphql():
    current_user = get_jwt_identity()
    result = graph_client.raw_execute(client, request.json)
    if result:
        return jsonify(data=result)
    return {
        "data": {},
        "errors": [{"message": "No results"}],
    }, 500


if THREATDB_DATA_DIR and os.path.exists(THREATDB_DATA_DIR):
    if os.access(THREATDB_DATA_DIR, os.R_OK):

        @filemon(THREATDB_DATA_DIR)
        def data_drop(signum):
            try:
                data_loader.start(client, THREATDB_DATA_DIR, remove_on_success=True)
            except Exception:
                LOG.debug(
                    f"Error processing the files in {THREATDB_DATA_DIR}. This is usually due to duplicate invocations."
                )

    else:
        LOG.warn(
            f"API server does not have read access to the directory {THREATDB_DATA_DIR}"
        )

    if not os.access(THREATDB_DATA_DIR, os.W_OK):
        LOG.warn(
            f"API server does not have write access to the directory {THREATDB_DATA_DIR}. Any processed file must be therefore removed via other means to avoid duplication."
        )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import functools

# import orjson
from urllib.parse import quote_plus

import grpc
import pydgraph

from threat_db.logger import LOG

# from threat_db.schema import bom, component, vulns


def catch_db_errors(fn):
    @functools.wraps(fn)
    def caller(*args, **kwargs):
        try:
            return fn(*args, **kwargs)
        except grpc.RpcError as re:
            LOG.error(
                "Unable to connect to the database. Ensure the hostname is valid and supports grpc connectivity"
            )
        except Exception as ex:
            LOG.exception(ex)

    return caller


# Create a client stub.
@catch_db_errors
def create_client_stub(host, api_key=None):
    if api_key:
        return pydgraph.DgraphClientStub.from_cloud(host, api_key)
    # Handle standalone and localhost invocations
    if ":9080" in host:
        return pydgraph.DgraphClientStub(host)
    creds = grpc.ssl_channel_credentials()
    call_credentials = grpc.metadata_call_credentials(
        lambda context, callback: callback((("authorization", api_key),), None)
    )
    composite_credentials = grpc.composite_channel_credentials(creds, call_credentials)
    return pydgraph.DgraphClientStub(
        host, composite_credentials, options=(("grpc.enable_http_proxy", 0),)
    )


# Create a client.
def create_client(client_stub):
    return pydgraph.DgraphClient(client_stub)


# Drop All - discard all data and start from a clean slate.
@catch_db_errors
def drop_all(client):
    if client:
        return client.alter(pydgraph.Operation(drop_all=True))
    return None


# @catch_db_errors
# def create_schemas(client):
#     client.alter(pydgraph.Operation(schema=component))
#     client.alter(pydgraph.Operation(schema=bom))
#     client.alter(pydgraph.Operation(schema=vulns))


# @catch_db_errors
# def create_data(client, data):
#     txn = client.txn()
#     try:
#         return txn.mutate(set_obj=data, commit_now=True)
#     finally:
#         txn.discard()


# @catch_db_errors
# def create_bom_data(client, bom):
#     txn = client.txn()
#     try:
#         query = """query all($serialNumber:string) {
#             all(func: eq(serialNumber, $serialNumber)) {
#                 C as uid
#                 serialNumber
#             }
#         }
#         """
#         variables = {"$serialNumber": bom.get("serialNumber")}
#         bom["uid"] = "uid(C)"
#         bmutation = txn.create_mutation(set_obj=bom)
#         request = txn.create_request(
#             mutations=[bmutation], query=query, variables=variables, commit_now=True
#         )
#         response = txn.do_request(request)
#         return response
#     finally:
#         txn.discard()


# @catch_db_errors
# def create_component_data(client, comp):
#     txn = client.txn()
#     try:
#         query = """query all($purl:string) {
#             all(func: eq(purl, $purl)) {
#                 C as uid
#                 name
#                 purl
#             }
#         }
#         """
#         variables = {"$purl": comp.get("purl")}
#         comp["uid"] = "uid(C)"
#         cmutation = txn.create_mutation(set_obj=comp)
#         request = txn.create_request(
#             mutations=[cmutation], query=query, variables=variables, commit_now=True
#         )
#         response = txn.do_request(request)
#         return response
#     finally:
#         txn.discard()


# @catch_db_errors
# def create_vuln_data(client, vuln):
#     txn = client.txn()
#     try:
#         # Look for the component based on purl and link it to the vulnerability
#         query = """query all($purl:string) {
#             all(func: eq(purl, $purl)) {
#                 C as uid
#                 name
#                 purl
#             }
#         }
#         """
#         variables = {"$purl": vuln.get("purl")}
#         vuln["components"] = [
#             {
#                 "uid": "uid(C)",
#                 "dgraph.type": "Component",
#             }
#         ]
#         vmutation = txn.create_mutation(set_obj=vuln)
#         request = txn.create_request(
#             mutations=[vmutation],
#             query=query,
#             variables=variables,
#             commit_now=True,
#         )
#         response = txn.do_request(request)
#         # Add the vulnerability to the component
#         if response.json:
#             comp_json = orjson.loads(response.json)
#             comp_all = comp_json.get("all", [])
#             uids = response.uids
#             for cc in comp_all:
#                 if cc.get("uid"):
#                     uid_key = f"uid({vuln.get('id')})"
#                     cmutation = txn.create_mutation(
#                         set_obj={
#                             "vulnerabilities": [
#                                 {
#                                     "uid": f"{uids.get(uid_key)}",
#                                     "dgraph.type": "Vulnerability",
#                                     **vuln,
#                                 }
#                             ],
#                         }
#                     )
#                     resp = txn.create_request(
#                         query="""{
#                             q(func: eq(uid, $uid)) {
#                                 uid
#                                 name
#                             }
#                         }""",
#                         variables={"$uid": cc["uid"]},
#                         mutations=[cmutation],
#                         commit_now=True,
#                     )
#         return response
#     finally:
#         txn.discard()


def get(host, api_key=None):
    client_stub = create_client_stub(host, api_key)
    client = create_client(client_stub)
    return client_stub, client


def close(client_stub):
    if client_stub:
        client_stub.close()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import argparse
import os

import threat_db.client as db_client
import threat_db.graphclient as graph_client
import threat_db.loader as data_loader
from threat_db.logger import LOG


def build_args():
    """
    Constructs command line arguments for the threat_db
    """
    parser = argparse.ArgumentParser(
        prog="threat_db",
        description="Modern threat database.",
    )
    parser.add_argument(
        "--dgraph-host",
        dest="dgraph_grpc_host",
        default=os.getenv("DGRAPH_RPC_HOST", "localhost:9080"),
        help="DB hostname.",
    )
    parser.add_argument(
        "--graphql-host",
        dest="graphql_host",
        default=os.getenv("DGRAPH_GRAPHQL_HOST", "http://localhost:8080"),
        help="DB hostname.",
    )
    parser.add_argument(
        "--data-dir",
        dest="data_dir",
        default=os.getenv("THREATDB_DATA_DIR"),
        help="Directory to load data from.",
    )
    parser.add_argument(
        "--remove-on-success",
        action="store_true",
        default=False,
        dest="remove_on_success",
        help="Delete data file that are successfully imported.",
    )
    return parser.parse_args()


def main():
    args = build_args()
    _, client = graph_client.get(args.graphql_host, os.getenv("DGRAPH_API_KEY"))
    if args.data_dir:
        LOG.info(f"Importing data from {args.data_dir}")
        try:
            data_loader.start(
                client, args.data_dir, remove_on_success=args.remove_on_success
            )
        except Exception as ex:
            LOG.exception(ex)

    if client:
        client.close_sync()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import argparse
import os
import uuid
from datetime import datetime

from itsdangerous.url_safe import URLSafeSerializer

import threat_db.graphclient as graph_client
from threat_db.logger import LOG


def build_args():
    """
    Constructs command line arguments for the threat_db
    """
    parser = argparse.ArgumentParser(
        prog="threat_db_admin",
        description="Administration functions for threat database. Only to be used for local development purposes.",
    )
    parser.add_argument(
        "--dgraph-host",
        dest="dgraph_grpc_host",
        default=os.getenv("DGRAPH_RPC_HOST", "localhost:9080"),
        help="DB hostname.",
    )
    parser.add_argument(
        "--graphql-host",
        dest="graphql_host",
        default=os.getenv("DGRAPH_GRAPHQL_HOST", "http://localhost:8080"),
        help="DB hostname.",
    )
    parser.add_argument(
        "--init",
        action="store_true",
        default=False,
        dest="init",
        help="Initialize database by dropping all data and recreating the schema.",
    )
    parser.add_argument(
        "--create-root-user",
        action="store_true",
        default=False,
        dest="create_root_user",
        help="Create the initial administrator user.",
    )
    return parser.parse_args()


def create_first_user(client):
    password = URLSafeSerializer(
        secret_key=os.urandom(32).hex(), salt="activate"
    ).dumps(24)
    email = "admin@localhost"
    user_id = str(uuid.uuid4())
    team_id = str(uuid.uuid4())
    role_id = str(uuid.uuid4())
    result = graph_client.create_user(
        client,
        [
            {
                "id": user_id,
                "email": email,
                "password": str(password),
                "fullName": "Local Admin",
                "created": datetime.now().isoformat(),
                "modified": datetime.now().isoformat(),
                "teams": [
                    {
                        "id": team_id,
                        "name": "Default",
                        "description": "Default team",
                        "tags": ["default", "dev"],
                        "users": [{"id": user_id}],
                        "created": datetime.now().isoformat(),
                        "modified": datetime.now().isoformat(),
                        "disabled": False,
                    }
                ],
                "roles": [
                    {
                        "id": role_id,
                        "user": {"id": user_id},
                        "team": {"id": team_id},
                        "role": "Administrator",
                        "created": datetime.now().isoformat(),
                        "modified": datetime.now().isoformat(),
                        "disabled": False,
                    }
                ],
                "disabled": False,
            }
        ],
    )
    return {
        "result": result,
        "user_id": user_id,
        "email": email,
        "password": password,
    }


def main():
    args = build_args()
    _, client = graph_client.get(args.graphql_host, os.getenv("DGRAPH_API_KEY"))
    if args.init:
        try:
            if graph_client.is_alive(client, args.graphql_host):
                LOG.info("Dropping all data and recreating db schemas")
                graph_client.drop_all(client, args.graphql_host)
                graph_client.create_schemas(client, args.graphql_host)
            else:
                LOG.info(f"Database is not live yet. Check the DB logs for any issues.")
        except Exception:
            LOG.warn(
                "Unable to create schemas due to issues connecting to the database."
            )
    if args.create_root_user:
        try:
            LOG.info("Creating a new admin user for development purposes")
            create_res = create_first_user(client)
            if create_res:
                # Check if authentication works for the new user
                user_id = create_res["user_id"]
                password = create_res["password"]
                auth_res = graph_client.auth_user(client, user_id, password)
                if auth_res:
                    LOG.info(
                        f"An administrator user was created with the id {user_id} and password {password}"
                    )
                    LOG.info(
                        """Use this credential for development purposes only and ensure this account is removed in production.\nTo generate access token for this user, make a POST request to the /login endpoint\n\nexport ACCESS_TOKEN=$(curl -q -X POST %(graphql_host)s/login -d '{"username":"%(user_id)s","password":"%(password)s"}' -H "Content-Type: application/json" | jq -r '.access_token')"""
                        % dict(
                            user_id=user_id,
                            password=password,
                            graphql_host=args.graphql_host,
                        )
                    )
                else:
                    LOG.info(f"Unable to authenticate as the new user {user_id}")
            else:
                LOG.warn(
                    "Unable to create the first administrator user. Check for any errors in the logs."
                )
        except Exception as ex:
            LOG.exception(ex)
            LOG.warn(
                "Unable to create user due to issues connecting to the database. Check if the schema was created successfully."
            )



import os
import re
from tempfile import SpooledTemporaryFile

import orjson

import threat_db.graphclient as graph_client
from threat_db.logger import LOG
from threat_db.utils import find_files, parse_purl, safe_remove

lic_symbol_regex = re.compile(r"[\(\)\,]")


def cleanup_license_string(license_str):
    """
    Method to cleanup license string by removing problematic symbols and making certain keywords consistent
    :param license_str: String to clean up
    :return: Cleaned up version
    """
    if not license_str:
        license_str = ""
    license_str = (
        license_str.replace(" / ", " OR ")
        .replace("/", " OR ")
        .replace(" & ", " OR ")
        .replace("&", " OR ")
    )
    license_str = lic_symbol_regex.sub("", license_str)
    return license_str.upper()


def get_pkg_vulns_json(jsonfile):
    """Method to extract packages from a bom json file"""
    if not os.path.exists(jsonfile):
        return {}
    with open(jsonfile) as fp:
        try:
            bom_data = orjson.loads(fp.read())
            if bom_data:
                return get_pkg_vulns_from_bom(bom_data)
        except Exception:
            return {}


def get_pkg_vulns_from_bom(bom_data):
    components = []
    serial_number = None
    metadata = {}
    metadata = bom_data.get("metadata")
    vulnerabilities = []
    added_vkeys = {}
    if bom_data.get("components"):
        serial_number = bom_data.get("serialNumber", "")
        for comp in bom_data.get("components"):
            licenses = []
            vendor = comp.get("group")
            if not vendor:
                vendor = ""
            if comp.get("licenses"):
                for lic in comp.get("licenses"):
                    license_obj = lic
                    # licenses has list of dict with either license or expression as key
                    # Only license is supported for now
                    if lic.get("license"):
                        license_obj = lic.get("license")
                    if license_obj.get("id"):
                        licenses.append(license_obj.get("id"))
                    elif license_obj.get("name"):
                        licenses.append(cleanup_license_string(license_obj.get("name")))
            purl = comp.get("purl")
            type = ""
            subpath = ""
            qualifiers = {}
            repo_url = ""
            download_url = ""
            if purl:
                purl_obj = parse_purl(purl)
                type = purl_obj.get("type", "")
                subpath = purl_obj.get("subpath", "")
                qualifiers = purl_obj.get("qualifiers", {})
                repo_url = purl_obj.get("repo_url", "")
                download_url = purl_obj.get("download_url", "")
            fcomp = {
                **comp,
                "isRoot": False,
                "bomRef": comp.get("bom-ref"),
                "ctype": type,
                "subPath": subpath,
                "repoUrl": repo_url,
                "downloadUrl": download_url,
                "qualifiers": qualifiers,
                "vendor": vendor,
                "licenses": licenses,
                "appearsIn": [{"serialNumber": serial_number}],
            }
            del fcomp["bom-ref"]
            components.append(fcomp)
        for avuln in bom_data.get("vulnerabilities", []):
            bomRef = avuln.get("bom-ref")
            if added_vkeys.get(bomRef):
                continue
            affects = []
            version = ""
            fix_version = ""
            severity = "none"
            cvss_score = 0
            for ac in avuln.get("affects", []):
                affects.append({"purl": ac.get("ref")})
                for ver in ac.get("versions"):
                    if ver.get("status") == "affected":
                        version = ver.get("version")
                    if ver.get("status") == "unaffected":
                        fix_version = ver.get("version")
            if avuln.get("ratings"):
                for ar in avuln.get("ratings"):
                    if ar.get("method") == "CVSSv31":
                        severity = ar.get("severity")
                        cvss_score = ar.get("score")
            fvuln = {
                **avuln,
                "bomRef": bomRef,
                "affects": affects,
                "version": version,
                "fix_version": fix_version,
                "severity": severity,
                "cvss_score": cvss_score,
            }
            del fvuln["bom-ref"]
            vulnerabilities.append(fvuln)
            added_vkeys[bomRef] = True
    return {
        "serial_number": serial_number,
        "metadata": metadata,
        "components": components,
        "services": bom_data.get("services", []),
        "vulnerabilities": vulnerabilities,
        "bom_data": bom_data,
    }


def process_vex(client, data_dir, remove_on_success=False):
    json_files = find_files(data_dir, ".vex.json", False, True)
    for jsonf in json_files:
        success = process_vex_file(client, jsonf)
        if success and remove_on_success:
            safe_remove(jsonf)


def process_vex_file(client, jsonf):
    parsed_obj = {}
    if isinstance(jsonf, SpooledTemporaryFile):
        try:
            parsed_obj = get_pkg_vulns_from_bom(orjson.loads(jsonf.read()))
        except Exception as ex:
            LOG.warn("Exception while converting to json from tempfile")
            LOG.exception(ex)
            return False
    else:
        LOG.debug(f"Processing {jsonf}")
        parsed_obj = get_pkg_vulns_json(jsonf)
    serial_number = parsed_obj.get("serial_number")
    components = parsed_obj.get("components")
    metadata = parsed_obj.get("metadata")
    services = parsed_obj.get("services")
    if serial_number and components:
        LOG.info(
            f"Creating Bom with {len(components)} components and {len(services)} services from {jsonf}"
        )
        root_component = metadata.get("component", None)
        if root_component and root_component.get("purl"):
            root_component["isRoot"] = True
            root_component["bomRef"] = root_component.get("bom-ref")
            root_component["ctype"] = root_component.get("type")
            del root_component["bom-ref"]
        result = graph_client.create_bom(
            client,
            [
                {
                    "serialNumber": serial_number,
                    "metadata": {
                        "timestamp": metadata.get("timestamp").replace("Z", ""),
                        "component": root_component,
                    },
                    "components": components,
                    "services": services,
                    "vulnerabilities": parsed_obj.get("vulnerabilities"),
                }
            ],
        )
        if result and result.get("addBom"):
            LOG.debug(result)
            return True
        else:
            return False
    return True


def start(client, data_dir, remove_on_success=False):
    process_vex(client, data_dir, remove_on_success)



drop_all_query = """
    {
        "drop_all": true
    }
"""

introspect_query = """
    query IntrospectionQuery {
        __schema {
            types {
            name
            }
        }
    }
"""

health_query = """
    query {
        health {
            instance
            address
            version
            status
            lastEcho
            group
            uptime
            ongoing
            indexing
        }
    }
"""

auth_user_query = """
    query ($user_id: String!, $password: String!) {
        checkUserPassword(id: $user_id, password: $password) {
            id
            email
        }
    }
"""



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['threat_db']

package_data = \
{'': ['*']}

install_requires = \
['flask-jwt-extended>=4.4.4,<5.0.0',
 'flask>=2.2.2,<3.0.0',
 'gql[all]>=3.4.0,<4.0.0',
 'grpcio>=1.51.1,<2.0.0',
 'itsdangerous>=2.1.2,<3.0.0',
 'orjson>=3.8.5,<4.0.0',
 'packageurl-python>=0.10.4,<0.11.0',
 'protobuf>=3.20.1,<4.0.0',
 'pydgraph>=21.3.2,<22.0.0',
 'rich>=13.3.1,<14.0.0',
 'uwsgi>=2.0.21,<3.0.0']

entry_points = \
{'console_scripts': ['threat_db = threat_db.cli:main',
                     'threat_db_admin = threat_db.admincli:main']}

setup_kwargs = {
    'name': 'threat-db',
    'version': '0.6.3',
    'description': 'A graphql server for vulnerabilities powered by dgraph',
    'long_description': '# Introduction\n\nThreatDB is a graph database for application components and vulnerabilities powered by dgraph. Currently, CycloneDX 1.4 SBoM and VEX files could be imported and queried with this project.\n\n## Development setup\n\n```\ngit clone https://github.com/appthreat/threat-db.git\ncd threat-db\nmkdir -p $HOME/dgraph $HOME/threatdb_data_dir\ndocker compose up\n```\n\nThis would start a threat db api server (PORT: 9000) and an instance of [dgraph](https://dgraph.io) standalone (PORTS: 8080, 9080).\n\n## Create schemas\n\nTo create the schemas and the first administrator user.\n\n```\ngit clone https://github.com/appthreat/threat-db.git\npip install poetry\npoetry install\nexport DGRAPH_API_KEY=changeme\npoetry run threat_db_admin --init --dgraph-host localhost:9080 --graphql-host http://localhost:8080/graphql\npoetry run threat_db_admin --create-root-user --dgraph-host localhost:9080 --graphql-host http://localhost:8080/graphql\n```\n\nCopy the user id and password from the logs.\n\n## Import data\n\n```\nmkdir -p $HOME/threatdb_data_dir\nthreat_db --data-dir $HOME/threatdb_data_dir\n```\n\nWhen invoked with docker compose, any .vex.json files present in the directory `THREATDB_DATA_DIR` would be imported automatically. For testing purposes, you can download some sample VEX files from [here](https://github.com/appthreat/images-info/actions/workflows/build.yml)\n\n## Rest API\n\n### Generate access token\n\n```\ncurl -X POST http://0.0.0.0:9000/login -d "username=user id&password=password" -H "Content-Type: application/json"\n```\n\nUseful one-liner for automation\n\n```\nexport ACCESS_TOKEN=$(curl -X POST http://0.0.0.0:9000/login -d \'{"username":"username","password":"password"}\' -H "Content-Type: application/json" | jq -r \'.access_token\')\n```\n\n```\ncurl http://0.0.0.0:9000/healthcheck\n```\n\n### whoami\n\n```\ncurl http://0.0.0.0:9000/whoami -H "Authorization: Bearer $ACCESS_TOKEN"\n```\n\n### Import data\n\n```\ncurl -F \'file=@/tmp/bom.json\' http://0.0.0.0:9000/import -H "Authorization: Bearer $ACCESS_TOKEN"\n```\n\n## Cloud Setup\n\nRefer to the instructions under [contrib](contrib/microk8s/INSTALL.md) to setup a microk8s cluster with threat-db and dgraph.\n\n## Discord support\n\nThe developers could be reached via the [discord](https://discord.gg/DCNxzaeUpd) channel.\n',
    'author': 'Team AppThreat',
    'author_email': 'cloud@appthreat.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': '',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<3.11',
}


setup(**setup_kwargs)



# Default ignore list
ignore_directories = [
    ".git",
    ".svn",
    ".mvn",
    ".idea",
    "dist",
    "bin",
    "obj",
    "backup",
    "docs",
    "tests",
    "test",
    "tmp",
    "node_modules",
    ".terraform",
    ".serverless",
    "venv",
    "examples",
    "tutorials",
    "samples",
    "migrations",
    "db_migrations",
    "unittests",
    "unittests_legacy",
    "stubs",
    "mock",
    "mocks",
]

# JWT expiry period - 90 days
JWT_ACCESS_TOKEN_EXPIRES_HOURS = 90 * 24



import os

from packageurl import PackageURL
from packageurl.contrib import purl2url

import threat_db.config as config


def filter_ignored_dirs(dirs):
    """
    Method to filter directory list to remove ignored directories
    :param dirs: Directories to ignore
    :return: Filtered directory list
    """
    [
        dirs.remove(d)
        for d in list(dirs)
        if d.lower() in config.ignore_directories or d.startswith(".")
    ]
    return dirs


def find_files(src, src_ext_name, quick=False, filter=True):
    """
    Method to find files with given extenstion
    """
    result = []
    for root, dirs, files in os.walk(src):
        if filter:
            filter_ignored_dirs(dirs)
        for file in files:
            if file == src_ext_name or file.endswith(src_ext_name):
                result.append(os.path.join(root, file))
                if quick:
                    return result
    return result


def parse_purl(purl_str):
    """Method to parse a package url string safely"""
    try:
        purl_obj = PackageURL.from_string(purl_str).to_dict() if purl_str else {}
        if purl_obj:
            purl_obj["repo_url"] = purl2url.get_repo_url(purl_str)
            purl_obj["download_url"] = purl2url.get_download_url(purl_str)
        return purl_obj
    except ValueError:
        tmpA = purl_str.split("@")[0]
        purl_obj = {}
        if tmpA:
            tmpB = tmpA.split("/")
            if tmpB:
                if len(tmpB) < 2:
                    purl_obj["name"] = tmpB[-1].lower()
                    purl_obj["namespace"] = tmpB[0].split(":")[-1]
                if len(tmpB) > 2:
                    namespace = tmpB[-2]
                    if tmpB[-2].startswith("pkg:"):
                        namespace = tmpB[-2].split(":")[-1]
                    purl_obj["namespace"] = namespace
        return purl_obj


def safe_remove(f):
    try:
        os.remove(f)
    except Exception as e:
        pass
